"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveOperations = void 0;
const formatName_1 = require("../filters/format/formatName");
const formatOperation_1 = require("../filters/format/formatOperation");
/**
 * This function tries to resolve the Operations of a given OpenAPI document.
 * @param api The OpenAPI document for which to resolve the Operations.
 * @returns The resolved Operations.
 */
function resolveOperations(api) {
    const simplifiedOperations = [];
    if (api.paths)
        simplifiedOperations.push(...parsePaths(api.paths));
    return simplifiedOperations;
}
exports.resolveOperations = resolveOperations;
function parsePaths(paths) {
    const simplifiedOperations = [];
    for (const path of Object.keys(paths)) {
        const pathItem = paths[path];
        simplifiedOperations.push(...parsePathItem(path, pathItem));
    }
    return simplifiedOperations;
}
function parsePathItem(path, pathItem) {
    const simplifiedOperations = [];
    for (const method in pathItem) {
        const operation = pathItem[method];
        simplifiedOperations.push(parseOperation(path, method, operation));
    }
    return simplifiedOperations;
}
function parseOperation(path, method, operation) {
    // Create SimplifiedOperation with known properties
    const simplifiedOperation = {
        name: (0, formatOperation_1.formatOperation)(path, method),
        path: path,
        method: method,
        serviceName: operation['x-service-name'],
        operationId: operation.operationId ?? '',
        security: operation.security,
        summary: operation.summary ?? '',
        external: operation['x-internal'] ? false : true,
        destructureInput: operation['x-destructure-input'] ?? false,
        buildUrl: operation['x-build-url'] ?? false,
        optionalUrl: operation['x-optional-url'] ?? false,
        isProxyRequest: operation['x-proxy-request'] ?? false,
    };
    // Search for parameters to add
    const parameters = operation.parameters;
    if (parameters)
        parseParameters(parameters, simplifiedOperation);
    // Search for requestBody to add
    const requestBody = operation.requestBody;
    if (requestBody)
        parseRequestBody(requestBody, simplifiedOperation);
    // Search for responses to add
    const responses = operation.responses;
    if (responses)
        parseResponses(responses, simplifiedOperation);
    return simplifiedOperation;
}
function parseParameters(parameters, simplifiedOperation) {
    for (const parameter of parameters) {
        simplifiedOperation.parameters ??= [];
        const simplifiedParameter = {
            name: parameter.name,
            required: parameter.required ?? false,
            in: parameter.in,
            description: parameter.description,
            schema: parameter.schema,
        };
        simplifiedOperation.parameters.push(simplifiedParameter);
    }
}
function parseRequestBody(requestBody, simplifiedOperation) {
    simplifiedOperation.requestBody = {
        description: requestBody.description,
        required: requestBody.required ?? false,
    };
    if (requestBody.content && requestBody.content['application/json'].schema) {
        simplifiedOperation.requestBody.schema =
            requestBody.content['application/json'].schema;
    }
}
function parseResponses(responses, simplifiedOperation) {
    for (const status in responses) {
        const response = responses[status];
        simplifiedOperation.responses ??= [];
        const simplifiedResponse = {
            status: status,
            description: response.description,
        };
        // Add schema of response if present
        if (response.content && response.content['application/json'].schema) {
            simplifiedResponse.schema = response.content['application/json'].schema;
        }
        // Add headers of response if present
        if (response.headers)
            parseHeaders(response.headers, simplifiedResponse);
        simplifiedOperation.responses.push(simplifiedResponse);
    }
}
function parseHeaders(headers, simplifiedResponse) {
    simplifiedResponse.headers = [];
    for (const headerName in headers) {
        const header = headers[headerName];
        simplifiedResponse.headers.push({
            name: (0, formatName_1.formatName)(headerName),
            required: header.required ?? false,
            schema: header.schema,
        });
    }
}
//# sourceMappingURL=resolveOperations.js.map