"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateTyping = void 0;
const formatName_1 = require("../filters/format/formatName");
const allOf_1 = require("./keywords/allOf");
const anyOf_1 = require("./keywords/anyOf");
const const_1 = require("./keywords/const");
const enum_1 = require("./keywords/enum");
const oneOf_1 = require("./keywords/oneOf");
const array_1 = require("./types/array");
const boolean_1 = require("./types/boolean");
const integer_1 = require("./types/integer");
const null_1 = require("./types/null");
const number_1 = require("./types/number");
const object_1 = require("./types/object");
const string_1 = require("./types/string");
function generateTyping(schema, options) {
    // set default values for options
    options ??= {};
    options.inline ??= false;
    options.resolveDirectly ??= true;
    options.context ??= [];
    options.schemaType ??= 'all';
    options.prefixDirectlyResolved ??= '';
    const comment = schema.description
        ? `/**\n * ${schema.description.replace(/\n/g, '\n * ')}\n */\n`
        : '';
    const contextSchema = options.context.find((ctxSchema) => ctxSchema.title === schema.title && ctxSchema['x-standalone']);
    // Handle subtype and different required properties
    if (options.resolveDirectly && schema.title && contextSchema) {
        return resolveSchemaDirectly({ ...schema, title: schema.title }, contextSchema, comment, options);
    }
    // Handle allOf
    if (schema.allOf)
        return (0, allOf_1.handleAllOf)(schema, comment, options);
    // Handle anyOf
    if (schema.anyOf)
        return (0, anyOf_1.handleAnyOf)(schema, comment, options);
    // Handle oneOf
    if (schema.oneOf)
        return (0, oneOf_1.handleOneOf)(schema, comment, options);
    // Handle enum
    if (schema.enum)
        return (0, enum_1.handleEnum)(schema, comment);
    // Handle const
    if (schema.const)
        return (0, const_1.handleConst)(schema, comment);
    // Handle the different possible types
    switch (schema.type) {
        case 'object':
            return (0, object_1.handleObject)(schema, comment, options);
        case 'array':
            return (0, array_1.handleArray)(schema, comment, options);
        case 'integer':
            return (0, integer_1.handleInteger)(comment);
        case 'boolean':
            return (0, boolean_1.handleBoolean)(comment);
        case 'string':
            return (0, string_1.handleString)(comment);
        case 'null':
            return (0, null_1.handleNull)(comment);
        case 'number':
            return (0, number_1.handleNumber)(comment);
    }
    return {
        typeDeclaration: 'any',
        typeDependencies: [],
        comment: comment,
    };
}
exports.generateTyping = generateTyping;
function resolveSchemaDirectly(schema, contextSchema, comment, options) {
    // check if schema has new required properties compared to
    // contextSchema and if so add them via the Require type
    let prefix = '';
    let suffix = '';
    if (options.context) {
        if (schema.required) {
            const contextRequired = contextSchema.required ?? [];
            const newRequired = schema.required.filter((required) => !contextRequired.includes(required));
            if (newRequired.length > 0) {
                prefix = 'Require<';
                suffix = `, ${newRequired.map((r) => `"${r}"`).join(' | ')}>`;
            }
        }
    }
    return {
        typeDeclaration: prefix +
            options.prefixDirectlyResolved +
            (0, formatName_1.formatName)(schema.title) +
            (options.schemaType === 'request' ? '<"request">' : '') +
            (options.schemaType === 'response' ? '<"response">' : '') +
            suffix,
        typeDependencies: [(0, formatName_1.formatName)(schema.title)],
        comment: comment,
    };
}
//# sourceMappingURL=typing.js.map